#pragma once

#include <unordered_map>
#include "connection.hpp"
#include "worker_node/src/primitive.hpp"
#include "worker_node/src/Codec.hpp"
#include "timers/src/timer_periodic.hpp"
#include "timers/src/timer_manager.hpp"

namespace cpm::client
{
    using address = std::string;
    using port = std::string;
}

namespace connections
{
    using tcp = boost::asio::ip::tcp;

    namespace interfaces
    {
        INTERFACE_S(handler,
                    INTERFACE_SERVER(void, notifyPrimitive, const primitive::GenericPtr &);)
        INTERFACE_C(handler,
                    INTERFACE_CLIENT(void, notifyPrimitive, const primitive::GenericPtr &);)

    } // namespace interfaces

    class manager : public std::enable_shared_from_this<manager>,
                    public interfaces::controller::server,
                    public interfaces::handler::client
    {
    public:
        explicit manager(const TypeInst &ownInst,
                         boost::asio::io_context &ioc);
        ~manager() override = default;

        void start(const std::string &listener,
                   const std::string &port);
        void stop(void);

        void send(primitive::GenericPtr prim) const;
        void send(const Id &id, primitive::GenericPtr prim) const;

    protected:
        void listenConnections(const std::string &listener,
                               const std::string &port = "0");

        void handle_brokenConnection(const TypeInst &inst) override;
        void handle_connectionClosed(const TypeInst &inst) override;

        TypeInst m_nodeInst;            ///< Node instance type.
        boost::asio::io_context &m_ioc; ///< Reference to context for asynchronous operations.

    private:
        connection_ptr getConnection(TypeInst inst) const;
        connection_ptr getConnection(const Id &id) const;

        void start_accept();
        void stop_accept();
        void on_accept(connection_ptr conn_p,
                       const boost::system::error_code &err);

        void handle_whoami(connection_ptr conn_ptr,
                           std::shared_ptr<primitive::generic> prim);

        void handleMessage(connection_ptr conn_ptr,
                           const primitive::raw_data_t &message) override;

        mutable std::recursive_mutex m_guard;                               ///< Protector of shared data in multiple threads.
        boost::asio::ip::tcp::acceptor m_acceptor;                          ///< Acceptor used to listen for incoming connections.
        bool m_stopped{false};                                              ///< Indicates stopping to listen to incoming connections.
        int m_connIdCntr{0};                                                ///< monotonic counter to create connection ID.
        std::unordered_map<Id, connection_ptr> m_connections;               ///< Connections storage.
        std::unordered_map<TypeInst, connection_ptr> m_instToConnectionMap; ///< Mapping connection per instance of destination node.
    };

}
