#pragma once

// AFAP includes.
#include "components/db/src/client.hpp"
#include "components/db/src/types.hpp"

// STL includes.
#include <memory>
#include <map>

namespace db
{
    namespace stub
    {
        namespace uvr
        {
            using RecordPtr = std::shared_ptr<::db::uvr::record>;
            using IpMap = std::unordered_multimap<Uvr::Ip_t, RecordPtr>;
            using Idmap = std::unordered_map<Uvr::UvrId_t, RecordPtr>;
        }

        namespace hub
        {
            using map = std::map<Uvr::Gt_t, ::db::hub::record>;
        }

        namespace Operator
        {
            using Map = std::map<Uvr::OperatorId_t, ::db::Operator::Record>;
        }

        namespace subs
        {
            using vector = ::db::subscriber::rows;
        }
        class mimic : public client
        {
        private:
            Uvr::UvrId_t m_uvrId;                   ///< ID of own VFN
            uvr::RecordPtr m_vfnInfoPtr{nullptr};   ///< Information about own VFN
            std::recursive_mutex m_incidentsGuard;  ///< Protector of shared data in multiple threads
            std::recursive_mutex m_incidentsAGuard; ///< Protector of shared data in multiple threads
            ::db::incident::rows m_incidents;       ///< The storage of incidents
            ::db::incidentA::rows m_incidentsA;     ///< The storage of incidents

            std::mutex m_uvrsByIdMutex;
            uvr::Idmap m_uvrsById;                  ///< The storage of UVR records per UVR ID

            std::mutex m_uvrsByIpMutex;
            uvr::IpMap m_uvrsByIp;                  ///< The storage of UVR records per primary or secondary IP

            std::mutex m_hubsMutex;
            hub::map m_hubs;                        ///< The storage of HUBs records

            std::mutex m_operatorsMutex;
            Operator::Map m_operators;

        public:
            explicit mimic(Uvr::UvrId_t uvrId);

            bool connect(void) override;
            void disconnect(void) override{/* Not used */};

            // tables
            void create(const ::db::table::name &tbName,
                        const ::db::table::fields &tbFields,
                        const ::db::table::settings &tbSettings) override{/* Not used */};
            void drop(const ::db::table::name &tbName) override{/* Not used */};
            void recreate(const ::db::table::name &tbName,
                          const ::db::table::fields &tb_fields,
                          const ::db::table::settings &tbSettings) override;
            void fill_in(const ::db::table::name &tbName,
                         const std::string &filename) override;
            void subscribe_on(const ::db::table::name &tbName) override{/* Not used */};
            ::db::limit_t size(const ::db::table::name &tbName) override;
            bool is_exist(const ::db::table::name &tbName) override;

            // calls
            void put(const Uvr::CallRecord &rec) override{/* Not used */};
            bool update(const Uvr::CallRecord &rec) override {/* Not used */ return true; };
            void get(Uvr::CallInformationResponses &rec,
                     const Uvr::CallInformationRequest &req) override{/* Not used */};

            // incidents
            void get(::db::incident::rows &rec,
                     const timers::time_point &start,
                     const timers::time_point &stop) override;
            void put(const ::db::incident::record &rec) override;
            // incidentsA
            void get(::db::incidentA::rows &rec,
                     const timers::time_point &start,
                     const timers::time_point &stop) override;
            void put(const ::db::incidentA::record &rec) override;
            // subscribers
            void populate(const ::db::subscriber::rows &rows) override{/* Not used */};
            void getall(const std::function<void(::db::subscriber::record &&)> &fn) override{/* Not used */};
            void get(::db::subscriber::rows &rows,
                     ::db::limit_t first,
                     ::db::limit_t last) override{/* Not used */};
            void put(const ::db::subscriber::record &rec) override{/* Not used */};
            void update(const ::db::subscriber::record &) override{/* Not used */};
            void remove(const Uvr::Number &) override{/* Not used */};
            // uvrs
            void populate(const ::db::uvr::rows &rows) override{/* Not used */};
            bool get(::db::uvr::record &rec,
                     const Uvr::UvrId_t &uvrId) override;
            void get(::db::uvr::rows &rows,
                     const Uvr::Ip_t &ipAddress) override;
            void put(const ::db::uvr::record &rec) override{/* Not used */};
            // hubs
            void populate(const ::db::hub::rows &rows) override{/* Not used */};
            void get(::db::hub::rows &rows,
                     const ::db::gts_t &list) override;
            void put(const ::db::hub::record &rec) override{/* Not used */};

            // operators
            void populate(const ::db::Operator::Rows &rows) override {/* Not used */};
            void get(::db::Operator::Rows &rows,
                     const ::db::OperatorsIds &list) override;
            void put(const ::db::Operator::Record &rec) override {/* Not used */};
        };

    } // namespace stub

} // namespace db
