#pragma once

// Log includes.
#include "Logger.hpp"

// STL includes.
#include <fstream>

namespace Log
{
    class FileLogger : public Logger
    {
    public:
        FileLogger(const std::string& filePath);
        ~FileLogger() override;

    private:
        bool openFile(const std::string& filePath);

    private:
        void processMessage(std::string &&message) override;

    private:
        std::string m_filePath;
        std::ofstream m_fileStream;
    };
} // namespace Log

//------------------------------------------------------------------------------
// Definitions
//------------------------------------------------------------------------------

namespace Log
{
    inline FileLogger::FileLogger(const std::string& filePath)
        : m_filePath(filePath)
    {
        if (!openFile(m_filePath))
        {
            // TODO[Log]: throw exception?
        }
    }

    inline FileLogger::~FileLogger()
    {
        m_fileStream << std::flush;
        if (m_fileStream.is_open())
        {
            m_fileStream.close();
        }
    }

    inline bool FileLogger::openFile(const std::string& filePath)
    {
        // TODO[Log]: create directory?

        m_fileStream.open(filePath, std::ios_base::out | std::ios_base::app);
        if (!m_fileStream.is_open())
        {
            return false;
        }

        return true;
    }

    inline void FileLogger::processMessage(std::string &&message)
    {
        m_fileStream << message + '\n';
    }
} // namespace Log
