#pragma once

// STL includes.
#include <string>

namespace Log
{
    class Tag
    {
    public:
        Tag() = default;

        Tag(const Tag &other);
        Tag(Tag &&other) noexcept;

        explicit Tag(const char *string);
        explicit Tag(const std::string &string);
        explicit Tag(std::string &&string);

        virtual ~Tag() = default;

    public:
        Tag & operator=(const Tag &other);
        Tag & operator=(Tag &&other) noexcept;

        Tag & operator=(const char *string);
        Tag & operator=(const std::string &string);
        Tag & operator=(std::string &&string);

    public:
        operator const std::string &() const;

    public:
        const std::string & value() const;
        bool isEmpty() const;

    public:
        friend std::ostream & operator<<(std::ostream &oStream, const Tag &tag);

    private:
        std::string m_value = "";
    };
} // namespace Log

//------------------------------------------------------------------------------
// Definitions
//------------------------------------------------------------------------------

namespace Log
{
    inline Tag::Tag(const Tag &other)
        : m_value(other.m_value)
    {
    }

    inline Tag::Tag(Tag &&other) noexcept
        : m_value(std::move(other.m_value))
    {
    }

    inline Tag::Tag(const char *string)
        : m_value(string)
    {
    }

    inline Tag::Tag(const std::string &string)
        : m_value(string)
    {
    }

    inline Tag::Tag(std::string &&string)
        : m_value(std::move(string))
    {
    }

    inline Tag & Tag::operator=(const Tag &other)
    {
        m_value = other.m_value;
        return *this;
    }

    inline Tag & Tag::operator=(Tag &&other) noexcept
    {
        m_value = std::move(other.m_value);
        return *this;
    }

    inline Tag & Tag::operator=(const char *string)
    {
        m_value = string;
        return *this;
    }

    inline Tag & Tag::operator=(const std::string &string)
    {
        m_value = string;
        return *this;
    }

    inline Tag & Tag::operator=(std::string &&string)
    {
        m_value = std::move(string);
        return *this;
    }

    inline Tag::operator const std::string &() const
    {
        return m_value;
    }

    inline const std::string & Tag::value() const
    {
        return m_value;
    }

    inline bool Tag::isEmpty() const
    {
        return m_value.empty();
    }

    inline std::ostream & operator<<(std::ostream &oStream, const Tag &tag)
    {
        oStream << tag.m_value;
        return oStream;
    }
} // namespace Log
