#pragma once

#include <limits>
#include <iostream>

class identifier
{
    using value_t = uint64_t;

    static const value_t undef = 0;
    static const value_t min = 1;
    static const value_t max =
#ifndef NO_VALUE
        std::numeric_limits<value_t>::max();
#else
        NO_VALUE;
#endif

public:
    value_t m_value;

    identifier() : m_value(undef) {}
    explicit identifier(const value_t &val) : m_value(val) {}
    identifier(const identifier &obj) = default;

    identifier &operator=(const identifier &obj) = default;
    identifier &operator=(const value_t &obj)
    {
        m_value = obj;
        return *this;
    }

    identifier &operator++() // ++identifier
    {
        if (m_value == undef)
        {
            m_value = min;
        }
        else
        {
            if (m_value > (max - 1))
            {
                m_value = min; // Do overlap
            }
            else
            {
                m_value++;
            }
        }
        return *this;
    }

    identifier operator++(int) // identifier++
    {
        identifier temp(*this);
        if (m_value == undef)
        {
            m_value = min;
        }
        else
        {
            if (m_value > (max - 1))
            {
                m_value = min; // Do overlap
            }
            else
            {
                m_value++;
            }
        }
        return temp;
    }

    void init()
    {
        m_value = min;
    }

    bool undefined() const
    {
        return (m_value == undef);
    }

    void reset()
    {
        m_value = undef;
    }

    static bool valid(size_t val)
    {
        return (val < max);
    }

    bool operator==(const identifier &obj) const
    {
        return m_value == obj.m_value;
    }
    bool operator==(const value_t &obj) const
    {
        return m_value == obj;
    }

    bool operator!=(const identifier &obj) const
    {
        return !(this->operator==(obj));
    }
    bool operator!=(const value_t &obj) const
    {
        return !(this->operator==(obj));
    }

    bool operator<(const identifier &obj) const
    {
        return m_value < obj.m_value;
    }

    bool operator>(const identifier &obj) const
    {
        return m_value > obj.m_value;
    }
};

inline std::ostream &operator<<(std::ostream &os, const identifier &obj)
{
    os << obj.m_value;
    return os;
}

namespace std
{

    template <>
    struct hash<identifier>
    {
        std::size_t operator()(const identifier &id) const
        {
            return id.m_value;
        }
    };

}
