#pragma once

// AFAP includes.
#include "components/support/variadic.hpp"

// STL includes.
#include <memory>

#define PARAMS_F0(A) A a0
#define PARAMS_F(N, A) A a##N,
#define PARAMS(N, ...) CAT(FOREACH, N)(N, PARAMS_F, __VA_ARGS__)

#define ARGS_F0(A) a0
#define ARGS_F(N, A) a##N,
#define ARGS(N, ...) CAT(FOREACH, N)(N, ARGS_F, __VA_ARGS__)

#define INTERFACE_CLIENT(...) \
    _INTERFACE_CLIENT(DEC(DEC(NARG(__VA_ARGS__))), __VA_ARGS__)
#define _INTERFACE_CLIENT(N, _ret_type, _func_name, ...) \
    _ret_type _func_name(PARAMS(N, __VA_ARGS__)) const   \
    {                                                    \
        assert(m_server_p != nullptr);                   \
        m_server_p->_func_name(ARGS(N, __VA_ARGS__));    \
    }

#define INTERFACE_CLIENT1(...) \
    _INTERFACE_CLIENT1(DEC(DEC(NARG(__VA_ARGS__))), __VA_ARGS__)
#define _INTERFACE_CLIENT1(N, _ret_type, _func_name, ...) \
    _ret_type _func_name(PARAMS(N, __VA_ARGS__))          \
    {                                                     \
        if (not m_server->is_stopped())                   \
        {                                                 \
            m_server->_func_name(ARGS(N, __VA_ARGS__));   \
        }                                                 \
    }

#define INTERFACE_CLIENT_R(...) \
    _INTERFACE_CLIENT_R(DEC(DEC(NARG(__VA_ARGS__))), __VA_ARGS__)
#define _INTERFACE_CLIENT_R(N, _ret_type, _func_name, ...)   \
    _ret_type _func_name(PARAMS(N, __VA_ARGS__)) const       \
    {                                                        \
        assert(m_server_p != nullptr);                       \
        return m_server_p->_func_name(ARGS(N, __VA_ARGS__)); \
    }

#define INTERFACE_CLIENT_P(...) \
    _INTERFACE_CLIENT_P(DEC(DEC(NARG(__VA_ARGS__))), __VA_ARGS__)
#define _INTERFACE_CLIENT_P(N, _ret_type, _func_name, ...) \
    _ret_type _func_name(PARAMS(N, __VA_ARGS__))

#define INTERFACE_SERVER(...) \
    _INTERFACE_SERVER(DEC(DEC(NARG(__VA_ARGS__))), __VA_ARGS__)
#define _INTERFACE_SERVER(N, _ret_type, _func_name, ...) \
    virtual _ret_type _func_name(PARAMS(N, __VA_ARGS__)) = 0

#define INTERFACE_PROPERTY(_type, _name)\
    _type _name

#define INTERFACE_S(_name, ...)         \
    namespace _name                     \
    {                                   \
        struct server                   \
        {                               \
        private:                        \
            bool m_stopped;             \
                                        \
        public:                         \
            server() : m_stopped{false} \
            {                           \
            }                           \
            virtual ~server()           \
            {                           \
            }                           \
            void stop_server()          \
            {                           \
                m_stopped = true;       \
            }                           \
            bool is_stopped()           \
            {                           \
                return m_stopped;       \
            }                           \
            __VA_ARGS__                 \
        };                              \
    }

#define INTERFACE_C(_name, ...)                     \
    namespace _name                                 \
    {                                               \
        struct client : public proto_client<server> \
        {                                           \
            __VA_ARGS__                             \
        };                                          \
    }
#define INTERFACE_C1(_name, ...)                      \
    namespace _name                                   \
    {                                                 \
        struct client1 : public proto_client1<server> \
        {                                             \
            __VA_ARGS__                               \
        };                                            \
    }

template <class T>
class proto_client
{
protected:
    T *m_server_p{nullptr};

public:
    proto_client() = default;
    void bind(T *server_p)
    {
        m_server_p = server_p;
    }
    void bind_over(proto_client *client_p)
    {
        m_server_p = client_p->m_server_p;
    }
};

template <class T>
class proto_client1
{
protected:
    std::shared_ptr<T> m_server;

public:
    proto_client1() = default;
    void bind(std::shared_ptr<T> server)
    {
        m_server = server;
    }
    void bind_over(proto_client1 *client_p)
    {
        m_server = client_p->m_server;
    }
};
