#pragma once

// Boost includes.
#include <boost/asio/deadline_timer.hpp>
#include <boost/asio/io_service.hpp>
#include <boost/system/error_code.hpp>

namespace timers
{
    using DeadlineTimer_t = boost::asio::deadline_timer;
    using DeadlineTimerPtr_t = std::unique_ptr<DeadlineTimer_t>;
    using HandleTimer_t = std::function<void()>;

    class TimerTask
    {
    public:
        explicit TimerTask(DeadlineTimerPtr_t timer, const HandleTimer_t &handle);
        void operator()(const boost::system::error_code &err) const;

    private:
        DeadlineTimerPtr_t m_timer;
        HandleTimer_t m_handle;
    };

    void runDelayed(boost::asio::io_service &ioc, uint32_t duration, HandleTimer_t handle);

} // namespace timers
