#include "worker_node/src/primitive.hpp"

#define ID2STR(I) \
    case I:       \
        return #I

namespace primitive
{
    std::string id2str(uint16_t id)
    {
        switch (id)
        {
            ID2STR(WHOAMI);
            ID2STR(WORKER_STOP_REQ);
            ID2STR(GET_INCIDENTS_REQ);
            ID2STR(GET_INCIDENTS_RSP);
            ID2STR(GET_STATISTICS_REQ);
            ID2STR(GET_STATISTICS_RSP);
            ID2STR(PM_PROVIDER_REGISTER_REQ);
            ID2STR(GET_COUNTERS_REQ);
            ID2STR(GET_COUNTERS_RSP);
            ID2STR(UPLOAD_NUMBERS_IND);
            ID2STR(UPLOAD_NODES_IND);
            ID2STR(UPLOAD_OPERATORS_IND);
            ID2STR(OUTGOING_CALL_IND);
            ID2STR(VERIFY_CALL_QUERY);
            ID2STR(VERIFY_CALL_VRD);
            ID2STR(VERIFY_CALL_REQ);
            ID2STR(VERIFY_CALL_RSP);
            ID2STR(VERIFY_CALL_IND);
            ID2STR(VERIFY_CALL_CNF);
            ID2STR(NOTIFY_CALL_INFO_REQ);
            ID2STR(NOTIFY_CALL_INFO_RSP);
            ID2STR(VERIFY_REQ_ABORT);
            ID2STR(UPLOAD_PUBLIC_KEYS_IND);
            ID2STR(QUERY_UVR_IDS_REQ);
            ID2STR(QUERY_UVR_IDS_RSP);
            ID2STR(QUERY_UVR_INFO_REQ);
            ID2STR(QUERY_UVR_INFO_RSP);
            ID2STR(QUERY_HUB_INFO_REQ);
            ID2STR(QUERY_HUB_INFO_RSP);
            ID2STR(QUERY_OPERATOR_INFO_REQ);
            ID2STR(QUERY_OPERATOR_INFO_RSP);
            ID2STR(EXEC_CMD_REQ);
            ID2STR(EXEC_CMD_RSP);
            ID2STR(REGISTER_CMD_IND);
            ID2STR(STATUS_CHANGE_IND);
            default:
                return "UNKNOWN(" + std::to_string(id) + ")";
        }
    }

    generic::Header::Header(const uint16_t &_id,
                            const TypeInst &_from,
                            const TypeInst &_to,
                            const ulpKey_t &_ulpKey)
        : id(_id),
          from(_from),
          to(_to),
          ulpKey(_ulpKey)
    {
    }

    bool generic::Header::operator==(const Header &other) const
    {
        return (id == other.id && from == other.from && to == other.to && ulpKey == other.ulpKey);
    }

    generic::generic(const uint16_t &_id,
                     const TypeInst &_from,
                     const TypeInst &_to,
                     const ulpKey_t &_ulpKey)
        : m_header(std::make_shared<Header>(_id, _from, _to, _ulpKey))
    {
    }

    generic::generic(const uint16_t &_id)
        : m_header(std::make_shared<Header>(_id, TypeInst{}, TypeInst{}, 0))
    {
    }

    generic::generic()
        : m_header(std::make_shared<Header>(UNKNOWN, TypeInst{}, TypeInst{}, 0))
    {
    }

    // WHOAMI
    Whoami::Whoami() : generic(WHOAMI)
    {
    }

    // WORKER_STOP_REQ
    WorkerStopReq::WorkerStopReq() : generic(WORKER_STOP_REQ)
    {
    }

    // GET_INCIDENTS_REQ
    GetIncidentsReq::GetIncidentsReq() : generic(GET_INCIDENTS_REQ)
    {
    }

    // GET_INCIDENTS_RSP
    GetIncidentsRsp::GetIncidentsRsp(const std::string &filename)
        : generic(GET_INCIDENTS_RSP),
          filename(filename)
    {
    }

    // SEND_INCIDENTSA_IND
    GetIncidentsARsp::GetIncidentsARsp(const std::string &filename)
        : generic(SEND_INCIDENTSA_IND),
          filename(filename)
    {
    }

    // UPLOAD_NUMBERS_IND
    UploadNumbersInd::UploadNumbersInd(const std::string &numFile,
                                       const Uvr::Strings &deltas)
        : generic(UPLOAD_NUMBERS_IND),
          numFile(numFile),
          deltas(deltas)
    {
    }

    // UPLOAD_NODES_IND
    UploadNodesInd::UploadNodesInd(const Uvr::Strings &files)
        : generic(UPLOAD_NODES_IND),
          files(files)
    {
    }

    // UPLOAD_OPERATORS_IND
    UploadOperatorsInd::UploadOperatorsInd(const Uvr::Strings &files)
        : generic(UPLOAD_OPERATORS_IND),
          files(files)
    {
    }

    // OUTGOING_CALL_IND
    OutgoingCallIndication::OutgoingCallIndication(Uvr::OctetStringPtr callingNumber,
                                                   Uvr::OctetStringPtr calledNumber)
        : generic(OUTGOING_CALL_IND),
          callingNumber(callingNumber),
          calledNumber(calledNumber)
    {
    }

    // VERIFY_CALL_QUERY
    VerifyCallQuery::VerifyCallQuery(const Uvr::OperatorId_t idSrc,
                                     const Uvr::OctetStringPtr &callingNumber,
                                     const Uvr::OctetStringPtr &calledNumber,
                                     const Uvr::OctetStringPtr &originalCalledNumber,
                                     const Uvr::OctetStringPtr &genericNumber,
                                     const Uvr::CallId_t callId)
        : generic(VERIFY_CALL_QUERY),
          idSrc(idSrc),
          callingNumber(callingNumber),
          calledNumber(calledNumber),
          originalCalledNumber(originalCalledNumber),
          genericNumber(genericNumber),
          callId(callId)
    {
    }

    // VERIFY_CALL_VERDICT
    VerifyCallVerdict::VerifyCallVerdict(Uvr::VerificationVerdict_t rlc,
                                         Uvr::UvrId_t uvrT)
        : generic(VERIFY_CALL_VRD),
          rlc(rlc),
          uvrT(uvrT)
    {
    }

    // VERIFY_CALL_REQUEST
    VerifyCallRequest::VerifyCallRequest(VerifyCallQueryPtr verifyCallQuery,
                                         Uvr::UvrId_t uvrP,
                                         Uvr::UvrId_t uvrS)
        : generic(VERIFY_CALL_REQ),
          verifyCallQuery(verifyCallQuery),
          uvrP(uvrP),
          uvrS(uvrS)
    {
    }

    // VERIFY_CALL_RESPONSE
    VerifyCallResponse::VerifyCallResponse(Uvr::VerificationVerdict_t verificationVerdict,
                                           Uvr::VerificationVerdict_t incidentVerdict,
                                           Uvr::UvrId_t uvrT)
        : generic(VERIFY_CALL_RSP),
          verificationVerdict(verificationVerdict),
          incidentVerdict(incidentVerdict),
          uvrT(uvrT)
    {
    }

    // VERIFY_CALL_IND
    VerifyCallIndication::VerifyCallIndication(ConnectionId_t connId,
                                               ServiceKey_t serviceKey,
                                               Uvr::UvrId_t uvrO,
                                               Uvr::UvrId_t uvrT,
                                               SessionKeyOptional_t sessionKey,
                                               Uvr::OctetStringPtr callingNumber,
                                               Uvr::OctetStringPtr calledNumber,
                                               Uvr::OctetStringPtr originalCalledNumber)
        : generic(VERIFY_CALL_IND),
          connId(connId),
          serviceKey(serviceKey),
          uvrO(uvrO),
          uvrT(uvrT),
          sessionKey(sessionKey),
          callingNumber(callingNumber),
          calledNumber(calledNumber),
          originalCalledNumber(originalCalledNumber)
    {
    }

    // VERIFY_CALL_CNF
    VerifyCallConfirm::VerifyCallConfirm(VerifyCallIndicationPtr verifyCallIndication,
                                         ReleaseCode_t releaseCode,
                                         AdditionalCode_t additionalCode,
                                         ErrorCode_t errorCode)
        : generic(VERIFY_CALL_CNF),
          verifyCallIndication(verifyCallIndication),
          releaseCode(releaseCode),
          additionalCode(additionalCode),
          errorCode(errorCode)
    {
    }

    // NOTIFY_CALL_INFO_REQ
    NotifyCallInfoRequest::NotifyCallInfoRequest(const Uvr::UvrId_t uvrId,
                                                 const Uvr::Strings &fileNamesList)
        : generic(NOTIFY_CALL_INFO_REQ),
          uvrId(uvrId),
          fileNamesList(fileNamesList)
    {
    }

    // NOTIFY_CALL_INFO_RSP
    NotifyCallInfoResponse::NotifyCallInfoResponse(const std::string &filename)
        : generic(NOTIFY_CALL_INFO_RSP),
          filename{filename}
    {
    }

    // VERIFY_REQ_ABORT
    VerifyRequestAbort::VerifyRequestAbort()
        : generic(VERIFY_REQ_ABORT)
    {
    }

    // UPLOAD_PUBLIC_KEYS_IND
    UploadPublicKeysInd::UploadPublicKeysInd(const Uvr::Strings &hubFilesToCopy,
                                             const Uvr::Strings &nodeFilesToCopy,
                                             const Uvr::Strings &hubFilesToRemove,
                                             const Uvr::Strings &nodeFilesToRemove)
        : generic(UPLOAD_PUBLIC_KEYS_IND),
          hubFilesToCopy(hubFilesToCopy),
          nodeFilesToCopy(nodeFilesToCopy),
          hubFilesToRemove(hubFilesToRemove),
          nodeFilesToRemove(nodeFilesToRemove)
    {
    }

    // QUERY_UVR_IDS_REQ
    QueryUvrIdsReq::QueryUvrIdsReq(const Uvr::OctetStringPtr &number)
        : generic(QUERY_UVR_IDS_REQ),
          number(number)
    {
    }

    // QUERY_UVR_IDS_RSP
    QueryUvrIdsRsp::QueryUvrIdsRsp(const uint16_t &primary, const uint16_t &secondary)
        : generic(QUERY_UVR_IDS_RSP)
    {
        uvrIds.primary   = primary;
        uvrIds.secondary = secondary;
    }

    //  QUERY_UVR_INFO_REQ
    QueryUvrInfoReq::QueryUvrInfoReq(const Uvr::UvrId_t &uvrId)
        : generic(QUERY_UVR_INFO_REQ),
          uvrId(uvrId)
    {
    }

    //  QUERY_UVR_INFO_RSP
    QueryUvrInfoRsp::QueryUvrInfoRsp(const ::db::uvr::record &rec)
        : generic(QUERY_UVR_INFO_RSP)
    {
        uvrInfo.id             = rec.m_idUvr;
        uvrInfo.gt             = rec.m_gtUvr;
        uvrInfo.ipPrimary      = rec.m_ipUvrP;
        uvrInfo.ipSecondary    = rec.m_ipUvrS;
        uvrInfo.dns            = rec.m_dnsUvr;
        uvrInfo.hubIdPrimary   = rec.m_idHubP;
        uvrInfo.hubIdSecondary = rec.m_idHubS;
        uvrInfo.gtUvr1         = rec.m_gtUvr1;
        uvrInfo.gtUvr2         = rec.m_gtUvr2;
        uvrInfo.idSrc          = rec.m_idSrc;
    }

    //  QUERY_HUB_INFO_REQ
    QueryHubInfoReq::QueryHubInfoReq(const Uvr::Gt_t &hubId)
        : generic(QUERY_HUB_INFO_REQ),
          hubId(hubId)
    {
    }

    //  QUERY_HUB_INFO_RSP
    QueryHubInfoRsp::QueryHubInfoRsp(const ::db::hub::record &rec)
        : generic(QUERY_HUB_INFO_RSP)
    {
        hubInfo.id           = rec.m_idHub;
        hubInfo.ipPrimary    = rec.m_ipHubP;
        hubInfo.ipSecondary  = rec.m_ipHubS;
        hubInfo.dnsPrimary   = rec.m_dnsHubP;
        hubInfo.dnsSecondary = rec.m_dnsHubS;
    }

    //  QUERY_OPERATOR_INFO_REQ
    QueryOperatorInfoReq::QueryOperatorInfoReq(const Uvr::OperatorId_t &operatorId)
        : generic(QUERY_OPERATOR_INFO_REQ),
          operatorId(operatorId)
    {
    }

    //  QUERY_OPERATOR_INFO_RSP
    QueryOperatorInfoRsp::QueryOperatorInfoRsp(const ::db::Operator::Record &rec)
        : generic(QUERY_OPERATOR_INFO_RSP)
    {
        operatorInfo.id       = rec.idSrc;
        operatorInfo.name     = rec.oprName;
        operatorInfo.nick     = rec.oprNick;
        operatorInfo.inn      = rec.inn;
        operatorInfo.bdpnCode = rec.bdpnCode;
        operatorInfo.brand    = rec.nameBrand;
    }

    //  EXEC_CMD_REQ
    ExecCmdReq::ExecCmdReq(const std::string &command,
                           const Uvr::Strings &commandArgs)
        : generic(EXEC_CMD_REQ),
          cmd(command),
          cmdArgs(commandArgs)
    {
    }

    //  EXEC_CMD_RSP
    ExecCmdRsp::ExecCmdRsp(Uvr::Strings &&responses)
        : generic(EXEC_CMD_RSP),
          responses(std::move(responses))
    {
    }

    // REGISTER_CMD_IND
    RegisterCmdInd::RegisterCmdInd(const std::string &command)
        : generic(REGISTER_CMD_IND),
          cmd(command)
    {
    }

    // STATUS_CHANGE_IND
    StatusChangeInd::StatusChangeInd(const Uvr::NodeStatus &status)
        : generic(STATUS_CHANGE_IND),
          status(status)
    {
    }

} // namespace primitive
