#include "components/worker_node/src/typeInst.hpp"

// STL includes.
#include <sstream>

TypeInst::TypeInst(uint8_t type, uint8_t inst)
{
    value = encode(type, inst);
}

uint8_t TypeInst::getType() const
{
    return ((value >> 2) & 0xF);
}

uint8_t TypeInst::getInstance() const
{
    return (value & 0x3);
}

bool TypeInst::operator==(const TypeInst &other) const
{
    return value == other.value;
}

bool TypeInst::operator!=(const TypeInst &other) const
{
    return !(*this == other);
}

TypeInst::operator std::string() const
{
    return nodeType2str(getType())
           + ":"
           + std::to_string(static_cast<int>(getInstance()));
}

bool TypeInst::operator<(const TypeInst &a) const
{
    return (value < a.value);
}

TypeInst::TypeInst(uint8_t _value)
    : value(_value)
{
}
TypeInst::TypeInst()
{
    value = encode(workers::ANY, 0);
}
std::string nodeType2str(const uint8_t &type)
{
    switch (type)
    {
    case workers::type::MANAGER:
        return "MANAGER";
    case workers::type::APIGW:
        return "APIGW";
    case workers::type::CCNGW:
        return "CCNGW";
    case workers::type::VFC:
        return "VFC";
    case workers::type::VFS:
        return "VFS";
    case workers::type::OAM:
        return "OAM";
    case workers::type::VFGW:
        return "VFGW";
    case workers::type::CLI:
        return "CLI";
    case workers::type::CLIGW:
        return "CLIGW";
    case workers::type::SIPMDL:
        return "SIPMDL";
    case workers::type::ANY:
        return "ANY";
    default:
        return "UNKWN(" + std::to_string(type) + ")";
    }
}

void tag_invoke(boost::json::value_from_tag, boost::json::value &jv, TypeInst const &c)
{
    jv = c.value;
}

TypeInst tag_invoke(boost::json::value_to_tag<TypeInst>, boost::json::value const &jv)
{
    return TypeInst{boost::json::value_to<uint8_t>(jv)};
}

std::ostream &operator<<(std::ostream &os, const TypeInst &ti)
{
    os << std::string(ti);
    return os;
}
