#pragma once

// AFAP includes.
#include "pm/provider/src/pm_node.hpp"
#include "oam/src/oam_pm_provider.hpp"
#include "oam/src/oam_types.hpp"

// STL includes.

namespace oam
{

    constexpr int PM_PROVIDER_MNGR_ID = 1;

    namespace CliCmds
    {
        const std::string printCounters = "print-counters";
    }

    class node : public ::pm::node
    {
    public:
        explicit node(const TypeInst &ownInst,
                      boost::asio::io_context &ioc,
                      const configuration::manager &config);
        void start() override;

    protected:
        void handleSpecificPrimitive(const primitive::GenericPtr &prim) override;
        void prepareCounters() override;

    private:
        void handle_pmProviderRegisterReq(const primitive::GenericPtr &prim);
        void delete_pmProvider(TypeInst inst);
        void handle_getStatisticsReq(const primitive::GenericPtr &prim,
                                     OutputFormat outputFormat = OutputFormat::Text);
        void handle_getCountersRsp(const primitive::GenericPtr &prim);
        void handle_execCmdReq(const primitive::GenericPtr &prim);

        void registerCliCommands();
        void handleCmdPrintCounters(const std::shared_ptr<primitive::ExecCmdReq> &req);
        void sendRsp(const std::shared_ptr<::pm::primitive::GetStatisticsRsp> &getStatRsp,
                     OutputFormat outputFormat = OutputFormat::Text);
        void sendExecCmdRsp(const TypeInst &to, Uvr::Strings &&rsps);

        mutable std::recursive_mutex m_guard;
        std::unordered_map<TypeInst, std::shared_ptr<oam::pm::provider>> m_pm_providers;
        pm::transaction::manager m_pm_trans_mngr;
    };

    using api = workers::api<node, workers::type::OAM, 1>;

} // namespace oam
