#pragma once

// AFAP includes.
#include "components/support/interfaces_template.hpp"
#include "components/timers/src/timer_manager.hpp"
#include "components/timers/src/timer_periodic.hpp"
#include "pm/common/src/pm_types.hpp"

namespace pm
{

    class job;

    namespace interfaces
    {

        INTERFACE_S(notifier,
                    INTERFACE_SERVER(void, notify_timeout, const ::pm::job &, const ::timers::time_point &);)
        INTERFACE_C(notifier,
                    INTERFACE_CLIENT(void, notify_timeout, const ::pm::job &, const ::timers::time_point &);)

    } // namespace interfaces

    class job : public std::enable_shared_from_this<job>,
                public interfaces::notifier::client,
                public ::timers::interfaces::manager::server
    {
    public:
        using period = timers::duration;

        job(boost::asio::io_context &ioc,
            const std::string &name,
            period duration,
            const counter::vectorFullIDs &counters);
        ~job() override;

        const std::string &name() const;
        const period &get_period() const;
        const counter::vectorFullIDs &counters() const;

        void start();
        void stop();

    protected:
        boost::asio::io_context &m_ioc;
        // Job name
        std::string m_name;
        // Collection period
        period m_period;
        // List of counters IDs
        counter::vectorFullIDs m_counters;

        std::shared_ptr<timers::periodic> m_timer;

        void manage_timeout(const ::timers::id &id,
                            const ::timers::time_point &start_time) override;
        void manage_stopped(const ::timers::id &id) override;
    };

} // namespace pm
