#include "UvrAPI.hpp"

#include "TRecQ763Number.hpp"

#include <iostream>

void verifyCallVerdictCallback(const Uvr::VerificationResult &result)
{
    // Do something
    return;
};

void callInformationRequestCallback(
    const Uvr::UvrId_t /* uvrId */,
    const Uvr::CallInformationRequest &request)
{
    Uvr::CallInformationResponse response(request.numA,
                                          request.numB,
                                          request.numC,
                                          request.numD,
                                          request.date,
                                          request.date,
                                          request.idReq,
                                          1,
                                          1,
                                          Uvr::ResponseCode::found);

    Uvr::API::reportCallInformation({response});
    return;
};

int main(int argc, char **argv)
{
    if (argc < 2)
    {
        std::cout << "No configuration file is provided\n";
        return 1;
    }

    std::string param1(argv[1]);

    if (param1 == "-v")
    {
        std::cout << Uvr::Troubleshoot::doVersions();
        return 0;
    }

    // Initialize
    if (!Uvr::API::init(param1))
    {
        return 1;
    }

    Uvr::UserId_t s_userId;
    s_userId = Uvr::API::registerVerifyCallVerdictCallback(verifyCallVerdictCallback);
    Uvr::API::registerCallInformationRequestCallback(callInformationRequestCallback, s_userId);

    // Start component
    Uvr::API::start();

    const Uvr::Number numberA("79876543210");
    const Uvr::Number numberB("79876543211");
    Uvr::OctetStringPtr numA = ::TRecQ763Number::toOctetStringPtr(numberA);
    Uvr::OctetStringPtr numB = ::TRecQ763Number::toOctetStringPtr(numberB);
    Uvr::CallId_t callId = "123-4567-890"; // Some unique ID for call
    
    auto operatorIdDst = (Uvr::OptionalOperatorId)2;
    Uvr::API::notifyAboutOutgoingCall(numA, numB, callId, operatorIdDst);

    auto trusted = false;
    auto operatorIdSrc = (Uvr::OperatorId_t)1;
    Uvr::SessionId_t transId =
        Uvr::API::verifyCall(s_userId,
                             operatorIdSrc,
                             numA,
                             numB,
                             Uvr::OctetStringPtr(),
                             Uvr::OctetStringPtr(),
                             callId,
                             trusted);

    /* Do something in scope of another thread
    /  Call Uvr::API::stop() when done
    */

    Uvr::API::waitStop();

    return 0;
}
