#include "components/log/src/Log.hpp"
#include "components/tools/src/Tools.hpp"

#include <string>
#include <vector>
#include <fstream>

#define REPLACE_MODE "replace"
#define ERASE_MODE "erase"

int main(int argc, char **argv)
{
    if (argc < 3)
    {
        LOGERROR("Wrong syntax: at least input and output files should be provided to edit!");
        return 1;
    }

    std::string filename(argv[1]);
    LOGDEBUG("File to edit:", filename);
    std::string outFilename(argv[2]);
    LOGDEBUG("Output file: ", outFilename);
    if (argc < 4)
    {
        LOGERROR("Wrong syntax: mode should be provided to edit!");
        return 1;
    }
    std::string mode(argv[3]);
    LOGDEBUG("Mode: ", mode);

    boost::json::value filesValue;
    filesValue.emplace_object();

    boost::json::value fileValue;
    if (Json::Errors::OK != Json::parseFile(filename, fileValue))
    {
        // error is printed inside function above
        return 1;
    }

    if (!fileValue.is_object())
    {
        LOGERROR("The content of file=\"", filename, " is not Json object!");
        return 1;
    }

    if (mode == REPLACE_MODE)
    {
        if (argc < 5)
        {
            LOGERROR("Wrong syntax: path to an object to edit should be provided!");
            return 1;
        }
        std::string path(argv[4]);
        LOGDEBUG("Path: ", path);

        if (argc < 6)
        {
            LOGERROR("Wrong syntax: an object to replace should be provided!");
            return 1;
        }
        std::string replacement(argv[5]);
        LOGDEBUG("Replacement: ", replacement);

        boost::json::value replacementValue;
        boost::json::error_code errorCode;

        replacementValue = boost::json::parse(replacement, errorCode);
        if (errorCode)
        {
            LOGERROR("Replacement:", replacement, " is not a valid Json object");
            return 2;
        }

        try
        {
            boost::json::value val = fileValue.at_pointer(path);
            LOGDEBUG("Value at path: ", path, " is ", val);
        }
        catch (const boost::json::system_error &ex)
        {
            LOGERROR("Value to replace is not found at ", path);
            return 2;
        }

        fileValue.set_at_pointer(path, replacementValue, errorCode);
        LOGINFO("Replaced at path: ", path, " with ", replacement);
    }
    else if (mode == ERASE_MODE)
    {
        if (argc < 5)
        {
            LOGERROR("Wrong syntax: path to an object to edit should be provided!");
            return 1;
        }
        std::string obj(argv[4]);
        LOGDEBUG("Obj: ", obj);

        if (argc < 6)
        {
            LOGERROR("Wrong syntax: a key inside the object to delete should be provided!");
            return 1;
        }
        std::string key(argv[5]);
        LOGDEBUG("Key: ", key);

        try
        {
            boost::json::value val = fileValue.at_pointer(obj);
            LOGDEBUG("Value to erase at ", key, " is ", val);
        }
        catch (const boost::json::system_error &ex)
        {
            LOGERROR("Object: ", obj, " to erase key: ", key, " with is not found");
            return 2;
        }

        boost::json::value &val = fileValue.at_pointer(obj);
        boost::json::object &valObject = val.get_object();
        auto it = valObject.find(key);
        if (it == valObject.end())
        {
            LOGERROR("Failed to find a key: ", key, " to erase with object:", obj);
            return 3;
        }
        valObject.stable_erase(it);
        LOGINFO("Erased key: ", key, " with object: ", obj);
    }

    // Put the content of all JSON files into one
    std::ofstream out(outFilename, std::ios::out | std::ios::trunc);
    if (!out.is_open())
    {
        LOGERROR("Cannot open output file=\"", outFilename, "\" for writting");
        return 1;
    }
    Json::prettyPrint(out, fileValue);
    out.close();
}
