#include "components/log/src/Log.hpp"
#include "components/tools/src/Tools.hpp"

#include <string>
#include <vector>
#include <fstream>

int main(int argc, char **argv)
{
    if (argc < 3)
    {
        LOGERROR("Wrong syntax: at least two input files should be provided to join!");
        return 1;
    }

    std::vector<std::string> filesToJoin;
    std::string outFilename;

    // skip program name, start from arguments
    for (int index = 1; index < argc; index++)
    {
        std::string arg(argv[index]);
        if (arg.compare("-o") != 0)
        {
            filesToJoin.push_back(arg);
            continue;
        }

        // next argument should be a name of output file
        // and it should be the last argument
        if ((index + 2) != argc)
        {
            LOGERROR("Wrong syntax: the name of output file should be the last argument!");
            return 1;
        }
        outFilename = std::string(argv[index + 1]);
        break;
    }
    // If output file is not specified, make it first file by default
    if (outFilename.empty())
    {
        outFilename = filesToJoin.front();
    }

#if LOG_MACRO_LEVEL >= LOG_LEVEL_DEBUG
    LOGDEBUG("Files to join:");
    for (auto const &filename : filesToJoin)
        LOGDEBUG("\t", filename);
    LOGDEBUG("Output file: ", outFilename);
#endif

    boost::json::value filesValue;
    filesValue.emplace_object();
    for (auto const &filename : filesToJoin)
    {
        boost::json::value fileValue;
        if (Json::Errors::OK != Json::parseFile(filename, fileValue))
        {
            // error is printed inside function above
            return 1;
        }

        if (!fileValue.is_object())
        {
            LOGERROR("The content of file=\"", filename, " is not Json object!");
            return 1;
        }

        boost::json::object fileObject = fileValue.get_object();
        for (auto it = fileObject.begin(); it != fileObject.end(); it++)
        {
            filesValue.get_object()[it->key()] = it->value();
        }
    }

    // Put the content of all JSON files into one
    std::ofstream out(outFilename, std::ios::out | std::ios::trunc);
    if (!out.is_open())
    {
        LOGERROR("Cannot open output file=\"", outFilename, "\" for writting");
        return 1;
    }
    Json::prettyPrint(out, filesValue);
    out.close();

    LOGDEBUG("Files were joined into one!");
}
