#pragma once

// AFAP includes.
#include "components/support/interfaces_template.hpp"
#include "components/timers/src/timer_types.hpp"
#include "components/worker_node/src/primitive.hpp"
#include "components/worker_node/src/transaction_manager.hpp"

// STL includes.
#include <list>

namespace vfc
{

    constexpr int VERIFY_REQ_MNGR_ID = 1;

    namespace transaction
    {
        namespace vf_req
        {
            struct watchdog
            {
                timers::time_point m_start;
                ::transaction::id m_transaction_id;
                bool m_is_active{false};

                watchdog();
            };

            class data : public ::transaction::base_data
            {
            public:
                Uvr::MetaInfo::BitFlags_t m_metaInfoFlags;
                uint16_t m_idUvrT;
                ::primitive::VerifyCallQueryPtr m_query;
                watchdog *m_watchdog;

                data() = default;
                data(const Uvr::MetaInfo::BitFlags_t metaInfoFlags,
                     const uint16_t &idUvrT,
                     ::primitive::VerifyCallQueryPtr query);
            };
        }
    }

    namespace interfaces
    {
        INTERFACE_S(verifier,
                    INTERFACE_SERVER(void, doTimeoutVerdict,
                                     transaction::vf_req::data &,
                                     ::transaction::id);)
        INTERFACE_C(verifier,
                    INTERFACE_CLIENT(void, doTimeoutVerdict,
                                     transaction::vf_req::data &,
                                     ::transaction::id);)
    }
    namespace transaction
    {
        namespace vf_req
        {
            class manager : public ::transaction::manager<data>,
                            public interfaces::verifier::client
            {
            public:
                manager(const TypeInst &ownInst,
                        ::transaction::manager_id mngrId,
                        Uvr::UvrId_t uvrId);

                ::transaction::id openTransaction(data &&value,
                                                   ::transaction::errors &result,
                                                   ::transaction::id key = ::transaction::undef_value) override;
                data closeTransaction(::transaction::id key,
                                       ::transaction::errors &result) override;
                void check_watchdogs();

            protected:
                std::list<watchdog> m_watchdogs;
            };
        }
    }

} // namespace vfc
