#pragma once

// AFAP includes.
#include "pm/provider/src/pm_node.hpp"
#include "vfs/src/registry.hpp"
#include "components/db/src/client.hpp"

namespace vfs
{
    class node : public pm::node,
                 public db::interfaces::tables::client,
                 public db::interfaces::tables::incidentsA::client
    {
    protected:
        void start_node_timers();

        void prepareCounters() override;

        void handleSpecificPrimitive(const primitive::GenericPtr &prim) override;
        void handle_timeout(const ::timers::id &id,
                            const ::timers::time_point &start_time) override;

        void getIncidentsA();
        void doIncident(const primitive::VerifyCallIndication &ind);
        void doIncident(const primitive::VerifyCallRequest &req);
        void doIncident(const Uvr::OctetStringPtr &callingNumber,
                        const Uvr::OctetStringPtr &calledNumber,
                        const Uvr::OctetStringPtr &originalCalledNumber,
                        const Uvr::UvrId_t &idUvrO);

        registry m_registry;
        timers::id m_registryWatchdogTimerId;
        timers::id m_incidentsATimerId;
        std::shared_ptr<::db::client> m_db_client_p;

    public:
        explicit node(const TypeInst &ownInst,
                      boost::asio::io_context &ioc,
                      const configuration::manager &config);
        void start(void) override;
    };

    using api = workers::api<node, workers::type::VFS, workers::max_num_of_insts>;

} // namespace vfs
