#pragma once

// AFAP includes.
#include "components/timers/src/timer_types.hpp"
#include "components/worker_node/src/primitive.hpp"

// STL includes.
#include <list>
#include <shared_mutex>

namespace vfs
{

    class registry
    {
    public:
        enum class CheckResult
        {
            pass,
            fail,
            timeout
        };

        registry() = default;

        void handleOutgoingCall(const primitive::OutgoingCallIndication &ind);
        CheckResult handleVerifyCall(const primitive::VerifyCallIndication &ind);
        CheckResult handleVerifyCall(const primitive::VerifyCallRequest &req);
        void do_sanity();
        void setCallTTL(const ::timers::time_point &callTTL);
        void setSearchTimeout(const ::timers::time_point &searchTimeout);

    private:
        struct call_info
        {
            Uvr::OctetStringPtr m_callingNumber;
            Uvr::OctetStringPtr m_calledNumber;
            ::timers::time_point m_start;

            call_info(Uvr::OctetStringPtr callingNumber,
                      Uvr::OctetStringPtr calledNumber)
                : m_callingNumber(callingNumber),
                  m_calledNumber(calledNumber)
            {
                timers::now(m_start);
            }
        };

        /// @brief Looks for an entry in the registry with matching numbers and
        ///        erases it if it was found.
        /// @param callingNumber Calling Number in TRecQ763 format.
        /// @param calledNumber  Called or Original Called Number in TRecQ763 format.
        /// @returns @c CheckResult::pass if such entry was registered,
        ///          @c CheckResult::fail if no call entry was matched,
        ///          @c CheckResult::timeout if search took too long.
        CheckResult wasRegistered(const Uvr::OctetStringPtr &callingNumber,
                                  const Uvr::OctetStringPtr &calledNumber,
                                  const timers::time_point &searchStart);
        CheckResult wasRegistered(const Uvr::OctetStringPtr &callingNumber,
                                  const Uvr::OctetStringPtr &calledNumber,
                                  const Uvr::OctetStringPtr &originalCalledNumber);

        std::list<call_info> m_calls;          ///< Outgoing calls storage
        mutable std::shared_mutex m_callGuard; ///< Protector of shared calls storage in multiple threads
        ::timers::time_point m_callTTL;
        ::timers::time_point m_searchTimeout;
    };

    std::ostream &operator<<(std::ostream &os, const registry::CheckResult &r);

} // namespace vfs
